package org.jboss.envers.demo.seam.persons;

import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.envers.VersionsReader;
import org.jboss.envers.query.VersionsRestrictions;

import javax.persistence.NoResultException;
import java.util.List;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.text.DateFormat;

/**
 * @author Adam Warski (adam at warski dot org)
 */
@Name("personsAddresses")
public class PersonsAddresses {
    @In
    private VersionsReader versionsReader;

    @In("#{personHome.instance}")
    private Person person;

    private List<Registration> registrations;

    private Date addressCheckDate;

    private Registration regOnCheckDate;

    private Registration regNow;

    private boolean checked;

    @SuppressWarnings({"unchecked"})
    public List<Registration> getRegistrations() {
        if (registrations == null) {
            registrations = versionsReader.createQuery()
                    .forRevisionsOfEntity(Registration.class, true, false)
                    .add(VersionsRestrictions.relatedIdEq("person", person.getId()))
                    .addOrder("registrationDate", true)
                    .getResultList();
        }

        return registrations;
    }

    public void check() {
        try {
            regOnCheckDate = (Registration) versionsReader.createQuery()
                    .forRevisionsOfEntity(Registration.class, true, false)
                    .add(VersionsRestrictions.maximizeProperty("actualDate")
                            .add(VersionsRestrictions.relatedIdEq("person", person.getId()))
                            .add(VersionsRestrictions.le("registrationDate", addressCheckDate))
                            .add(VersionsRestrictions.le("actualDate", addressCheckDate)))
                    .getSingleResult();
        } catch (NoResultException e) {
            regOnCheckDate = null;
        }

        try {
            regNow = (Registration) versionsReader.createQuery()
                    .forRevisionsOfEntity(Registration.class, true, false)
                    .add(VersionsRestrictions.maximizeProperty("actualDate")
                            .add(VersionsRestrictions.relatedIdEq("person", person.getId()))
                            .add(VersionsRestrictions.le("actualDate", addressCheckDate)))
                    .getSingleResult();
        } catch (NoResultException e) {
            regOnCheckDate = null;
        }

        checked = true;
    }

    public Date getAddressCheckDate() {
        return addressCheckDate;
    }

    public void setAddressCheckDate(Date addressCheckDate) {
        this.addressCheckDate = addressCheckDate;
    }

    private DateFormat addressCheckDateFormat = new SimpleDateFormat("d/MMM/y");

    public String getFormattedAddressCheckDate() {
        return addressCheckDateFormat.format(addressCheckDate);
    }

    public Registration getRegOnCheckDate() {
        return regOnCheckDate;
    }

    public void setRegOnCheckDate(Registration regOnCheckDate) {
        this.regOnCheckDate = regOnCheckDate;
    }

    public Registration getRegNow() {
        return regNow;
    }

    public void setRegNow(Registration regNow) {
        this.regNow = regNow;
    }

    public boolean isChecked() {
        return checked;
    }

    public void setChecked(boolean checked) {
        this.checked = checked;
    }
}
