package org.jboss.envers.demo.seam.persons;

import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.In;
import org.jboss.seam.faces.FacesMessages;
import org.jboss.envers.VersionsReader;
import org.jboss.envers.query.RevisionProperty;
import org.jboss.envers.query.VersionsRestrictions;

import javax.persistence.EntityManager;
import java.util.Date;

/**
 * @author Adam Warski (adam at warski dot org)
 */
@Name("register")
public class Register {
    private Date registrationDate;

    private Date actualDate;

    private Address address;

    @In("#{personHome.instance}")
    private Person person;

    @In
    private EntityManager entityManager;

    @In
    private VersionsReader versionsReader;

    @In
    private FacesMessages facesMessages;

    public Date getRegistrationDate() {
        return registrationDate;
    }

    public void setRegistrationDate(Date registrationDate) {
        this.registrationDate = registrationDate;
    }

    public Date getActualDate() {
        return actualDate;
    }

    public void setActualDate(Date actualDate) {
        this.actualDate = actualDate;
    }

    public Address getAddress() {
        return address;
    }

    public void setAddress(Address address) {
        this.address = address;
    }

    public boolean register() {
        // Checking if there is no registration with the same registration date already
        long registrationsWithSameRegDate = (Long) versionsReader.createQuery()
                .forRevisionsOfEntity(Registration.class, false, false)
                .addProjection(RevisionProperty.count())
                .add(VersionsRestrictions.relatedIdEq("person", person.getId()))
                .add(VersionsRestrictions.eq("registrationDate", registrationDate))
                .getSingleResult();

        if (registrationsWithSameRegDate != 0) {
            facesMessages.add("There already is a registration for that person with the given" +
                    " registration date!");
            return false;
        }

        // Checking if there is no registration with the same actual date already
        long registrationsWithSameActDate = (Long) versionsReader.createQuery()
                .forRevisionsOfEntity(Registration.class, false, false)
                .addProjection(RevisionProperty.count())
                .add(VersionsRestrictions.relatedIdEq("person", person.getId()))
                .add(VersionsRestrictions.eq("actualDate", actualDate))
                .getSingleResult();

        if (registrationsWithSameActDate != 0) {
            facesMessages.add("There already is a registration for that person with the given" +
                    " actual date!");
            return false;
        }

        // Storing the registration
        if (person.getRegistration() == null) {
            Registration registration = new Registration();

            registration.setPerson(person);
            registration.setActualDate(actualDate);
            registration.setRegistrationDate(registrationDate);
            registration.setAddress(address);

            entityManager.persist(registration);
        } else {
            Registration registration = person.getRegistration();

            registration.setActualDate(actualDate);
            registration.setRegistrationDate(registrationDate);
            registration.setAddress(address);
        }

        return true;
    }
}
